/**************************************************************/
/*                                                            */
/*   Program Name :  vex_ana                                  */
/*                                                            */
/*      Version:  1.10  2005-09-28                            */
/*                                                            */
/*  Copyright (c) 2003 T.Kondo/CRL All Right Reserved         */
/*                                                            */
/**************************************************************/
/*

  VEX file analysis sample program

          T.Kondo 2003-03-20
                  2005-09-28
                  2008-02-19  options '--help' and '--version'  were added 

   calling sequence
    
      m5vex_ana vex_file

          where  vex_file is VEX file
*/

#define PROGRAM_VERSION "Ver 1.11   2008-02-19"

#include "libk5.h"      // common header for IP-VLBI utilities

//#define DEBUG



int main(int argc, char *argv[])
// sample main program for VEX analysis
{
       int i,k;
       double telapsed;           // to measure processing time
       char sid[10]; // station ID (usually 2char)
       char mode[30];
       char cbuf[128];
       int nstation;
       //char* dflt_fname="G:/IPVLBI/data/evlbi3/2994.ovex";
       //char* dflt_fname="G:/IPVLBI/data/jiveexp/mk5.vex";
       char* dflt_fname="G:/IPVLBI/data/evlbi4/evlbi4.vex";
       vex_t vex;
       site_t site;
       eop_t eop;
       sidtable_t sidtable[10];
       track_freq_t track_freq;
	   time_t tstart, tfinish;    //           "



      if(argc == 1 || (argc == 2 && strncmp(*(argv+1),"--help",6)==0 )){ // 2008-02-19
          printf("\n");
          printf("m5vex_ana : %s\n",PROGRAM_VERSION);
          printf("\n");
          printf("Usage: m5vex_ana vex_file\n");
          printf("\n");
          printf("    where  vex_file ---- VEX file name\n");
          printf("\n");
          exit(0);
       }
       if(argc == 2 && strncmp(*(argv+1),"--ver",5)==0 ){ // 2008-02-19
          printf("\n");
          printf("m5vex_ana : %s\n",PROGRAM_VERSION);
          printf("\n");
          exit(0);
       } 

       /* data file name get */
       if (argc>=2){
          strcpy(vex.vex_file_name,*(argv+1));
       } else {
          strcpy(vex.vex_file_name,dflt_fname);
       }
       /* Program Name Display */
       printf("****************************************************\n");
       printf("*       VEX file analysis program (m5vex_ana)      *\n");
       printf("*      %s  by  T.KONDO/NICT     *\n",PROGRAM_VERSION);
       printf("****************************************************\n");
       printf("\n");

       tstart= time(NULL);
       printf("%s is opened!\n",vex.vex_file_name);
	   /* VEX file analysis */
       vex_read(&vex);
/*
       printf("Found $BLOCKs are\n");
       for(i=0; i<vex.nblock; i++) printf("%s ",vex.block[i]);
       printf("\n");
*/
       if(check_block(&vex)<0){
           printf("Warning! VEX file may be insuffient for correlation!\n");
       }

       // monitor schedule 
       //SKED_mon(&vex);

       printf("\nMODE found are\n");
       for(i=0;i<vex.nmode;i++){
           printf(" %d -- %s\n",i+1, vex.mode[i].def);
       }
       if(vex.nmode>1) {
          printf(" Enter Mode Number   ----> ");
          scanf("%s",&cbuf);
          i=atoi(cbuf);
       } else {
         i=1;
       }
       strcpy(mode,vex.mode[i-1].def);
       printf("\n");

       //create sid table

       create_siteid_table(&vex,&nstation,sidtable);
       printf("SITES (Station ID) defined are\n");
       for(i=0;i<nstation;i++){
           printf("  %d -- %s\n",i+1,sidtable[i].sid);
       }
       printf("Select Station by number   ----> ");
       scanf("%s",&cbuf);
       i=atoi(cbuf);
       //i=1;

       strcpy(sid,sidtable[i-1].sid);
       //strcpy(sid,"Wb");

       // search and get site info for sid
       if(search_site(&vex,sid,&site)==0){
          printf("\nDetailed site information\n");
          printf("   site definition :  %s\n",site.def);
          printf("   site name       :  %s\n",site.name);
          printf("   site ID         :  %s\n",site.id);
          printf("   site position   : %f %f %f\n",site.position[0],
                        site.position[1],site.position[2]);
          printf("   site clock\n");
          printf("      validity epoch : %d %d %d %d %d\n",
                   site.sclock.validity_epoch[0],
                   site.sclock.validity_epoch[1],
                   site.sclock.validity_epoch[2],
                   site.sclock.validity_epoch[3],
                   site.sclock.validity_epoch[4]);
          printf("      clock epoch    : %d %d %d %d %d\n",
                   site.sclock.clock_epoch[0],
                   site.sclock.clock_epoch[1],
                   site.sclock.clock_epoch[2],
                   site.sclock.clock_epoch[3],
                   site.sclock.clock_epoch[4]);
          printf("      clock offset   : %e\n",site.sclock.clock_offset);
          printf("      clock rate     : %e\n",site.sclock.clock_rate);

       }

       // search and get track and freq info  for sid
       printf("\nTRACK and FREQUENCY information for %s\n",site.name);
       if(search_track_freq(&vex,mode,sid,&track_freq)==0){
          printf("   BARREL ROLL : %s\n",track_freq.roll);
          printf("   FREQDEF = %s   TRACKDEF = %s\n",track_freq.freqdef,track_freq.trackdef);
          printf("   adbit= %d sample_rate= %f\n",track_freq.adbit, track_freq.samplrate);
          printf("\n");
          printf("   bb Tr   AD fo  chan     RF(Hz)        S     VBW(Hz)\n");
          for(i=0; i<track_freq.ntrack; i++){
              printf("   %2d %2d %5s %d %s %f %s %f\n",i+1, track_freq.trk[i].track,
                  track_freq.trk[i].bitstream,track_freq.trk[i].fan_out_pos,
                  track_freq.trk[i].chan_id,track_freq.trk[i].rf,
                  track_freq.trk[i].sideband,track_freq.trk[i].vbw);
          }
       }

       // found eop for experiment
       //    an example of how to get correct information
       if(vex.neop<1){
           printf("\nEOP information is not included in the file.\n");
       } else {
           for(i=0; i<vex.neop; i++){
             if(strncmp(vex.eop[i].def,vex.global.eop,strlen(vex.global.eop))==0) {  // MODE found
               eop=vex.eop[i];
               break;
             }
           }
           printf("\nEOP information\n");
           printf("   %s\n",eop.def);
           printf("   eop ref epoch   : ");
           for(k=0; k<5; k++) printf(" %d",eop.epoch[k]);
           printf("\n");
           printf("   eop_interval(hr): %d\n",eop.interval_hr);
           printf("   eop_ponts : %d\n",eop.points);
           i=eop.points;
           printf("   ut1-utc  : ");
           for(k=0; k<i; k++) printf(" %f",eop.ut1[k]);
           printf("\n");
           printf("   x_wobble : ");
           for(k=0; k<i; k++) printf(" %f",eop.xwob[k]);
           printf("\n");
           printf("   y_wobble : ");
           for(k=0; k<i; k++) printf(" %f",eop.ywob[k]);
           printf("\n");
       }
       printf("\n");

       //FREQ_mon(&vex);

       tfinish= time(NULL);
       telapsed = difftime(tfinish,tstart);
       //printf("Time elapsed (sec) for One obs process is %f\n",telapsed);


       return 0;
}
